from __future__ import print_function, division

__author__ = 'amrit'

import sys

#sys.dont_write_bytecode = True
import re, unicodedata
import nltk.corpus
import os
from nltk.stem import PorterStemmer
import pandas as pd

# numeric isolation
def numeric_isolation(x):
    return re.sub("[-+]?\d+[\.]?\d*", '', x)

# for words like don't, Adam's (Adam is)// We won't need is/not and would like to remove them
def special_case(x):
    x = re.sub(r"n't", ' not', x)
    return re.sub(r"'", '', x)

# all punctuation marks removal
def punctuate_preproc(x):
    return re.sub(r"<(.*?)>|\n|(\\(.*?){)|}|[#!$%^&*()_+|~\^\-<>/={}\[\],:\";<>?,.\/\\]|[@]", ' ', x)


# unicode normalisation
def unicode_normalisation(x):
    x = unicode(x, "utf-8")
    x = unicodedata.normalize('NFKD', x).encode('ascii', 'ignore')
    return x


# string lower
def string_lower(x):
    return x.lower()


# stemming
def stemming(x):
    port_stem = PorterStemmer()
    words = []
    for word in x.split(' '):
        words.append(port_stem.stem(word))
    return ' '.join(words)


# stopwords
def stopwords(x):
    temp = []
    stop = nltk.corpus.stopwords.words('english')
    x = re.findall(r"\b([a-zA-Z]+)\b", x)
    for i in x:
        if i not in stop:
            temp.append(i)
    return ' '.join(temp)



def str_len_less(x):
    if len(x) < 20:
        return ''
    else:
        return x


def word_len_less(x):
    line = re.findall(r"\b([a-zA-Z]+)\b", x)
    y = ''
    for word in line:
        if len(word) < 3:
            y = y + ''
        else:
            y = y + word + ' '
    return y

# only consider http and https not ftps
def email_urls(x):
    x = re.sub(r'\b[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[A-Za-z]{2,4}\b', '', x)
    return re.sub(
        r'(?i)\b((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}     /)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s!()\[\]{};:\'".,<>?]))',
        '', x)

# most steps need to be in specific order to achieve one
def process(x, *steps):
    for p in steps:
        x = p(x)
    return x

def run():
    file_names=[]
    for _, _, files in os.walk("../data/"):
        for file in files:
            if file.endswith(".csv"):
                file_names.append(file)

    for res in file_names:
        print(res)
        path = '../data/' + res
        path1 = '../data/preprocessed/'
        fo = open(path1 + res.split('.')[0] + '.txt', 'w')
        data = pd.read_csv(path)
        data['Description'] = data['Description'].apply(lambda x: str(x) + ' ')
        data['text'] = data['Description'] + data['Subject']
        labels=data['Severity'].values.tolist()
        li = data['text'].values.tolist()
        for i,doc in enumerate(li):
            line = process(doc, string_lower, punctuate_preproc,
                           numeric_isolation, stopwords, stemming, word_len_less)
            if len(line) > 2:
                fo.write(line + ">>> " + str(labels[i]))
                fo.write('\n')
        fo.close()

run()


from __future__ import print_function, division

__author__ = 'amrit'

import sys
#sys.dont_write_bytecode = True

from collections import OrderedDict, namedtuple
from random import random, randint, uniform, seed, choice, sample
import numpy as np

__all__ = ['DE']
Individual = namedtuple('Individual', 'ind fit')

class DE(object):
    def __init__(self, F=0.3, CR=0.7, NP=10, GEN=2, Goal="Max", termination="Early"):
        self.F=F
        self.CR=CR
        self.NP=NP
        self.GEN=GEN
        self.GOAL=Goal
        self.termination=termination
        seed(1)
        np.random.seed(1)

    def initial_pop(self):
        l=[]
        for _ in range(self.NP):
            dic=OrderedDict()
            for i in range(self.para_len):
                dic[self.para_dic.keys()[i]]=self.calls[i](self.bounds[i])
            l.append(dic)
        return l
        #return [{self.para_dic.keys()[i]:self.calls[i](self.bounds[i]) for i in range(self.para_len)} for _ in range(self.NP)]


    ## Need a tuple for integer and continuous variable but need the whole list for category
    def randomisation_functions(self):
        l=[]
        for i in self.para_category:
            if i=='integer':
                l.append(self._randint)
            elif i=='continuous':
                l.append(self._randuniform)
            elif i=='categorical':
                l.append(self._randchoice)
        self.calls=l

    ## Paras will be keyword with default values, and bounds would be list of tuples
    def solve(self, fitness, paras=OrderedDict(), bounds=[], category=[], **r):
        self.para_len=len(paras.keys())
        self.para_dic=paras
        self.para_category=category
        self.bounds=bounds
        self.randomisation_functions()
        initial_population=self.initial_pop()

        self.cur_gen = []
        for ind in initial_population:
            val = fitness(ind, **r)
            self.cur_gen.append(Individual(OrderedDict(ind), val[0]))

        if self.termination=='Early':
            return self.early_termination(fitness,**r)

        else:
            return self.late_termination(fitness,**r)

    def early_termination(self,fitness,**r):
        if self.GEN>1:
            for x in range(self.GEN):
                trial_generation = []
                for ind in self.cur_gen:
                    v = self._extrapolate(ind)
                    temp=fitness(v,**r)
#                    print(temp)
                    trial_generation.append(Individual(OrderedDict(v),temp[0] ))

                current_generation = self._selection(trial_generation)
                self.cur_gen=current_generation
            best_index = self._get_best_index()
            return self.cur_gen[best_index], self.cur_gen
        else:
            best_index = self._get_best_index()
            return self.cur_gen[best_index], self.cur_gen

    def late_termination(self,fitness,**r):
        lives=1
        while lives!=0:
            trial_generation = []
            for ind in self.cur_gen:
                v = self._extrapolate(ind)
                temp = fitness(v, **r)
                trial_generation.append(Individual(OrderedDict(v), temp[0]))
            current_generation = self._selection(trial_generation)
            if sorted(self.cur_gen)==sorted(current_generation):
                lives=lives-1
            else:
                self.cur_gen=current_generation

        best_index = self._get_best_index()
        return self.cur_gen[best_index], self.cur_gen

    def _extrapolate(self,ind):
        if (random() < self.CR):
            l = self._select3others()
            mutated=[]
            for x,i in enumerate(self.para_category):
                if i=='continuous':
                    mutated.append(l[0][l[0].keys()[x]]+self.F*(l[2][l[2].keys()[x]]-l[2][l[2].keys()[x]]))
                else:
                    mutated.append(self.calls[x](self.bounds[x]))
            check_mutated = []
            for i in range(self.para_len):
                if self.para_category[i]=='continuous':
                    check_mutated.append(max(self.bounds[i][0], min(mutated[i], self.bounds[i][1])))
                else:
                    check_mutated.append(mutated[i])
            dic=OrderedDict()
            for i in range(self.para_len):
                dic[self.para_dic.keys()[i]]=check_mutated[i]
            return dic
        else:
            dic = OrderedDict()
            for i in range(self.para_len):
                key=self.para_dic.keys()[i]
                dic[self.para_dic.keys()[i]] = ind.ind[key]
            return dic

    def _select3others(self):
        l=[]
        val=sample(self.cur_gen,3)
        for a in val:
            l.append(a.ind)
        return l

    def _selection(self, trial_generation):
        generation = []

        for a, b in zip(self.cur_gen, trial_generation):
            if self.GOAL=='Max':
                if a.fit >= b.fit:
                    generation.append(a)
                else:
                    generation.append(b)
            else:
                if a.fit <= b.fit:
                    generation.append(a)
                else:
                    generation.append(b)

        return generation

    def _get_best_index(self):
        if self.GOAL=='Max':
            best = 0
            max_fitness=-float("inf")
            for i, x in enumerate(self.cur_gen):
                if x.fit >= max_fitness:
                    best = i
                    max_fitness = x.fit
            return best
        else:
            best = 0
            max_fitness = float("inf")
            for i, x in enumerate(self.cur_gen):
                if x.fit <= max_fitness:
                    best = i
                    max_fitness = x.fit
            return best

    def _randint(self,a):
        return randint(*a)

    def _randchoice(self,a):
        return choice(a)

    def _randuniform(self,a):
        return uniform(*a)


# def main():
#     de=DE()
#     pop=de.solve(main,paras={'k':10,'x':5}, bounds=[(1,10),(1,10)],category=['integer','integer'])
#     print(pop)
# main()



from __future__ import print_function, division

__author__ = 'amrit'

import sys
from .demo import cmd
#sys.dont_write_bytecode = True
from collections import OrderedDict
import os
from collections import Counter
from .featurization import *
from .ML import DT, SVM, RF, FFT1
from sklearn.model_selection import StratifiedKFold
import pickle
import numpy as np
from random import seed, shuffle
import time

ROOT=os.getcwd()
files=["pitsA", "pitsB", "pitsC", "pitsD", "pitsE", "pitsF"]
MLS=[DT,RF, SVM]#,  FFT1]
MLS_para_dic=[OrderedDict([("min_samples_split",2),("min_impurity_decrease",0.0), ("max_depth",None),
                               ("min_samples_leaf", 1)]), OrderedDict([("min_samples_split",2),
                                ("max_leaf_nodes",None), ("min_samples_leaf",1), ("min_impurity_decrease",0.0),("n_estimators",10)]),
                   OrderedDict([("C", 1.0), ("kernel", 'linear'),
                                ("degree", 3)]), OrderedDict()]

metrics=['accuracy','recall','precision','false_alarm']
features=[TFIDF, HASHING]

def readfile1(filename=''):
    dict = []
    labels=[]
    with open(filename, 'r') as f:
        for doc in f.readlines():
            try:
                row = doc.lower().split(">>>")
                dict.append(row[0].strip())
                labels.append(row[1].strip())
            except:
                pass
    count=Counter(labels)
    import operator
    key=max(count.iteritems(), key=operator.itemgetter(1))[0]

    labels=map(lambda x: 1 if x == key else 0, labels)
    return np.array(dict), np.array(labels)

def _test(res=''):
    seed(1)
    np.random.seed(1)
    path=ROOT+"/../data/preprocessed/"+res+".txt"
    raw_data,labels=readfile1(path)
    temp={}

    for i in range(5):
        ranges=range(len(labels))
        shuffle(ranges)
        raw_data=raw_data[ranges]
        labels=labels[ranges]

        for fea in features:
            if fea.__name__ not in temp:
                temp[fea.__name__] = {}
            start_time = time.time()
            corpus,_=fea(raw_data)
            end_time = time.time() - start_time

            skf = StratifiedKFold(n_splits=5)
            for train_index, test_index in skf.split(corpus, labels):
                train_data, train_labels = corpus[train_index], labels[train_index]
                test_data, test_labels = corpus[test_index], labels[test_index]

                for j, le in enumerate(MLS):
                    if le.__name__ not in temp[fea.__name__]:
                        temp[fea.__name__][le.__name__]={}

                    start_time1 = time.time()
                    _,val=MLS[j](MLS_para_dic[j], train_data, train_labels, test_data, test_labels, 'recall')
                    end_time1 = time.time() - start_time1
                    for m in metrics:
                        if m not in temp[fea.__name__][le.__name__]:
                            temp[fea.__name__][le.__name__][m]=[]
                        temp[fea.__name__][le.__name__][m].append(val[0][m])

                    if 'times' not in temp[fea.__name__][le.__name__]:
                        temp[fea.__name__][le.__name__]['times']=[]
                    else:
                        temp[fea.__name__][le.__name__]['times'].append(end_time1 + end_time)
                    if fea.__name__ not in temp[fea.__name__][le.__name__]:
                        temp[fea.__name__][le.__name__]['features']=[]
                    else:
                        temp[fea.__name__][le.__name__]['features'].append(val[1])

    with open('../dump/untuned' +res+ '.pickle', 'wb') as handle:
        pickle.dump(temp, handle)

if __name__ == '__main__':
    eval(cmd ())


from __future__ import print_function, division

__author__ = 'amrit'

import sys

#sys.dont_write_bytecode = True
from sklearn.feature_extraction.text import HashingVectorizer,CountVectorizer,TfidfVectorizer
#import lda
from sklearn.decomposition import LatentDirichletAllocation

def TF(doc):
    vect= CountVectorizer(max_df=0.95, min_df=2,)
    X= vect.fit_transform(doc)
    return X.A,vect.get_feature_names()

def TFIDF(doc):
    vect= TfidfVectorizer()
    X= vect.fit_transform(doc)
    return X.A, vect.get_feature_names()

def HASHING(doc):
    vect= HashingVectorizer(n_features=1000)
    X= vect.fit_transform(doc)
    return X.A, ''

def LDA_(doc,**k):
    corpus, vocab=TF(doc)
    return LatentDirichletAllocation(max_iter=1000,learning_method='online',**k).fit_transform(corpus), vocab



from __future__ import print_function, division

__author__ = 'amrit'

from .LDADE import *
files=["pitsA", "pitsB", "pitsC", "pitsD", "pitsE", "pitsF"]
def get_top_words(model, path1, feature_names, n_top_words, i=0, file1=''):
    topics = []
    fo = open(path1+file1+'.txt', 'w+')
    fo.write("Run: " + str(i) + "\n")
    for topic_idx, topic in enumerate(model.components_):
        str1 = ''
        fo.write("Topic " + str(topic_idx) + ": ")
        for j in topic.argsort()[:-n_top_words - 1:-1]:
            str1 += feature_names[j] + " "
            fo.write(feature_names[j] + " ")
        str1=str(str1.encode('ascii', 'ignore'))
        topics.append(str1)
        fo.write("\n")
    fo.close()
    return topics

seed(1)
np.random.seed(1)
path1=ROOT+"/../results/"
for res in files:
    print(res)
    path=ROOT+"/../data/preprocessed/"+res+".txt"
    raw_data,_=readfile1(path)
    tf_vectorizer = CountVectorizer(max_df=0.95, min_df=2, stop_words='english')
    tf = tf_vectorizer.fit_transform(raw_data)

    lda1 = LatentDirichletAllocation(max_iter=1000, learning_method='online', n_components=10)

    lda1.fit_transform(tf)

    # print("done in %0.3fs." % (time() - t0))
    tf_feature_names = tf_vectorizer.get_feature_names()
    topics = get_top_words(lda1, path1, tf_feature_names, 8, i=0, file1=res)
    print(topics)



import setuptools

with open("README.md", "r") as fh:
        long_description = fh.read()

        setuptools.setup(
                    name="magician_package",
                    version="0.2.991",
                    author="RAISE_LAB",
                    author_email="smajumd3@ncsu.edu",
                    description="A small example package for text mining application with hyper-parameter optimization on multiple learning to automatically find best model",
                    long_description=long_description,
                    long_description_content_type="text/markdown",
                    url="https://github.com/ai-se/magician_package",
                    packages=setuptools.find_packages(),
                    classifiers=[
                        "Programming Language :: Python :: 2.7",
                        "License :: OSI Approved :: MIT License",
                        "Operating System :: OS Independent",
                    ],
        )
